'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function makeCancelable(getPromise) {
  let canceled = false;
  return {
    getPromise: () => new Promise((resolve, reject) => {
      getPromise().then(data => {
        if (!canceled) {
          resolve(data);
        }
      }).catch(error => {
        if (!canceled) {
          reject(error);
        }
      });
    }),
    cancel: () => {
      canceled = true;
    }
  };
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

const initialState = {
  completed: null,
  queued: []
};

function promiseStateReducer(state = initialState, action) {
  switch (action.type) {
    case "ADD":
      return _objectSpread2(_objectSpread2({}, state), {}, {
        queued: [...state.queued, action.promise]
      });

    case "CLEANUP":
      {
        const {
          promise
        } = action;

        if (state.queued.indexOf(promise) === -1) {
          return state;
        }

        promise.cancel();
        return _objectSpread2(_objectSpread2({}, state), {}, {
          queued: state.queued.filter(queuedPromise => queuedPromise !== promise)
        });
      }

    case "COMPLETE":
      {
        const queueIndex = state.queued.indexOf(action.promise); // If a newer promise finishes before older promises, cancel the older promises

        state.queued.slice(0, queueIndex).forEach(previousPromise => {
          previousPromise.cancel();
        });
        return _objectSpread2(_objectSpread2({}, state), {}, {
          queued: state.queued.slice(queueIndex + 1),
          completed: action.result
        });
      }

    case "RESET":
      return initialState;

    default:
      return state;
  }
}

function usePromiseStateReducer() {
  return react.useReducer(promiseStateReducer, initialState);
}

function usePromiseState() {
  const [{
    completed,
    queued
  }, dispatch] = usePromiseStateReducer();

  const status = (() => {
    if (queued.length) {
      return "PENDING";
    } else if (!completed) {
      return "INITIAL";
    }

    return completed.status;
  })();

  const state = [completed && completed.status === "ERROR" ? completed.value : null, completed && completed.status === "SUCCESS" ? completed.value : null, status];
  return [state, dispatch];
}

const defaultOptions = {
  mode: "latest",
  onCancel: undefined
};
function usePromise(getPromise, options = defaultOptions) {
  const [state, dispatch] = usePromiseState();
  const {
    mode = defaultOptions.mode,
    onCancel = defaultOptions.onCancel
  } = options;
  react.useEffect(() => {
    if (typeof getPromise === "undefined" || getPromise === null) {
      dispatch({
        type: "RESET"
      });
      return;
    }

    if (typeof getPromise !== "function") {
      throw new Error(`usePromise requires a function argument, but received ${typeof getPromise}`);
    } // Build an object containing a function to initiate the promise
    // as well as a function to cancel it. Use the customCancel arg
    // if it's provided.


    const promise = onCancel ? {
      getPromise,
      cancel: () => undefined
    } : makeCancelable(getPromise);
    dispatch({
      type: "ADD",
      promise
    });
    const instance = promise.getPromise();
    instance.then(result => {
      dispatch({
        type: "COMPLETE",
        promise,
        result: {
          status: "SUCCESS",
          value: result
        }
      });
    }).catch(error => {
      dispatch({
        type: "COMPLETE",
        promise,
        result: {
          status: "ERROR",
          value: error
        }
      });
    });

    if (typeof onCancel === "function") {
      promise.cancel = () => onCancel(instance);
    }

    return () => {
      if (mode === "latest") {
        dispatch({
          type: "CLEANUP",
          promise
        });
      }
    };
  }, [getPromise, dispatch, mode, onCancel]);
  return state;
}

exports.usePromise = usePromise;
//# sourceMappingURL=index.js.map
