import { useReducer, useEffect } from 'react';

function makeCancelable(getPromise) {
    let canceled = false;
    return {
        getPromise: () => new Promise((resolve, reject) => {
            getPromise()
                .then((data) => {
                if (!canceled) {
                    resolve(data);
                }
            })
                .catch((error) => {
                if (!canceled) {
                    reject(error);
                }
            });
        }),
        cancel: () => {
            canceled = true;
        },
    };
}

const initialState = {
    completed: null,
    queued: [],
};
function promiseStateReducer(state = initialState, action) {
    switch (action.type) {
        case "ADD":
            return {
                ...state,
                queued: [...state.queued, action.promise],
            };
        case "CLEANUP": {
            const { promise } = action;
            if (state.queued.indexOf(promise) === -1) {
                return state;
            }
            promise.cancel();
            return {
                ...state,
                queued: state.queued.filter((queuedPromise) => queuedPromise !== promise),
            };
        }
        case "COMPLETE": {
            const queueIndex = state.queued.indexOf(action.promise);
            // If a newer promise finishes before older promises, cancel the older promises
            state.queued.slice(0, queueIndex).forEach((previousPromise) => {
                previousPromise.cancel();
            });
            return {
                ...state,
                queued: state.queued.slice(queueIndex + 1),
                completed: action.result,
            };
        }
        case "RESET":
            return initialState;
        default:
            return state;
    }
}
function usePromiseStateReducer() {
    return useReducer(promiseStateReducer, initialState);
}

function usePromiseState() {
    const [{ completed, queued }, dispatch] = usePromiseStateReducer();
    const status = (() => {
        if (queued.length) {
            return "PENDING";
        }
        else if (!completed) {
            return "INITIAL";
        }
        return completed.status;
    })();
    const state = [
        completed && completed.status === "ERROR" ? completed.value : null,
        completed && completed.status === "SUCCESS" ? completed.value : null,
        status,
    ];
    return [state, dispatch];
}

const defaultOptions = {
    mode: "latest",
    onCancel: undefined,
};
function usePromise(getPromise, options = defaultOptions) {
    const [state, dispatch] = usePromiseState();
    const { mode = defaultOptions.mode, onCancel = defaultOptions.onCancel } = options;
    useEffect(() => {
        if (typeof getPromise === "undefined" || getPromise === null) {
            dispatch({ type: "RESET" });
            return;
        }
        if (typeof getPromise !== "function") {
            throw new Error(`usePromise requires a function argument, but received ${typeof getPromise}`);
        }
        // Build an object containing a function to initiate the promise
        // as well as a function to cancel it. Use the customCancel arg
        // if it's provided.
        const promise = onCancel
            ? {
                getPromise,
                cancel: () => undefined,
            }
            : makeCancelable(getPromise);
        dispatch({ type: "ADD", promise });
        const instance = promise.getPromise();
        instance
            .then((result) => {
            dispatch({
                type: "COMPLETE",
                promise,
                result: { status: "SUCCESS", value: result },
            });
        })
            .catch((error) => {
            dispatch({ type: "COMPLETE", promise, result: { status: "ERROR", value: error } });
        });
        if (typeof onCancel === "function") {
            promise.cancel = () => onCancel(instance);
        }
        return () => {
            if (mode === "latest") {
                dispatch({ type: "CLEANUP", promise });
            }
        };
    }, [getPromise, dispatch, mode, onCancel]);
    return state;
}

export { usePromise };
//# sourceMappingURL=index.js.map
